local screen = {}
local player = { x = 200, y = 0 }
local anim = 0.0
local vx = 0.0
local vy = 0.0
local balls = {}

function makeBall(x, y, radius)
  return {
    x = x,
    y = y,
    radius = radius,
    vx = 0.0,
    vy = 0.0
  }
end

function love.load()
  screen.w = love.graphics.getWidth()
  screen.h = love.graphics.getHeight()

  table.insert(balls, makeBall(220, 30, 20))
end

function love.update(dt)
  anim = anim + dt

  if love.keyboard.isDown('left') then vx = vx - 4000.0 * dt
  elseif love.keyboard.isDown('right') then vx = vx + 4000.0 * dt
  else
    if vx > 0.0 then
      vx = vx - 2000.0 * dt
    elseif vx < 0.0 then
      vx = vx + 2000.0 * dt
    end

    if math.abs(vx) < 10.0 then vx = 0.0 end
  end

  if vx > 800.0 then vx = 800.0 end
  if vx < -800.0 then vx = -800.0 end

  if player.x < 55.0 then
    if vx < 0.0 then
      vx = -vx
    end
  end

  if player.x > screen.w - 55.0 then
    if vx > 0.0 then
      vx = -vx
    end
  end

  if love.keyboard.isDown('down') then vy = vy - 4000.0 * dt
  elseif love.keyboard.isDown('up') then vy = vy + 4000.0 * dt
  else
    if vy > 0.0 then
      vy = vy - 2000.0 * dt
    elseif vy < 0.0 then
      vy = vy + 2000.0 * dt
    end

    if math.abs(vy) < 10.0 then vy = 0.0 end
  end

  if vy > 800.0 then vy = 800.0 end
  if vy < -800.0 then vy = -800.0 end

  player.x = player.x + vx * dt
  player.y = player.y + vy * dt

  if player.y < 0.0 then player.y = 0.0 end
  if player.y > screen.h - 50.0 then player.y = screen.h - 50.0 end

  for i = 1, #balls do
    local ball = balls[i]

    local dx = ball.x - player.x
    local dy = ball.y - (screen.h - 10.0 - player.y)
    local d = math.sqrt(dx * dx + dy * dy)

    if d <= 53.0 + ball.radius then
      local a = math.atan2(dy, dx)
      local xs = ball.vx + 1.0 * vx
      local ys = ball.vy + 1.0 * vy
      local speed = math.sqrt(xs * xs + ys * ys)
      ball.vx = speed * math.cos(a)
      ball.vy = speed * math.sin(a)
    end

    local wallFriction = 0.8
    if ball.x < ball.radius then ball.vx = wallFriction * math.abs(ball.vx) end
    if ball.x > screen.w - ball.radius then ball.vx = -wallFriction * math.abs(ball.vx) end
    if ball.y < ball.radius then ball.vy = wallFriction * math.abs(ball.vy) end
    if ball.y > screen.h - ball.radius then ball.vy = -wallFriction * math.abs(ball.vy) end

    ball.vy = ball.vy + 800.0 * dt
    ball.x = ball.x + ball.vx * dt
    ball.y = ball.y + ball.vy * dt
  end
end

function love.draw()
  love.graphics.clear()
  love.graphics.setColor(255, 255, 255)
  love.graphics.rectangle("fill", 0, 0, screen.w, screen.h)

  love.graphics.setColor(100, 100, 100)
  love.graphics.rectangle("fill", player.x - 20.0 - 3.0, screen.h - 15.0 - player.y, 4.0, player.y)
  love.graphics.rectangle("fill", player.x + 20.0 - 3.0, screen.h - 15.0 - player.y, 4.0, player.y)

  love.graphics.setColor(255, 255, 255)
  love.graphics.arc("fill", player.x, screen.h - 10.0 - player.y, 50.0, -math.pi, 0.0, 64)
  love.graphics.setColor(0, 0, 0)
  love.graphics.setLineWidth(6.0)
  love.graphics.arc("line", player.x, screen.h - 10.0 - player.y, 50.0, -math.pi, 0.0, 64)

  love.graphics.setColor(0, 0, 0)
  love.graphics.circle("fill", player.x - 20.0, screen.h - 15.0, 15.0, 64)
  love.graphics.circle("fill", player.x + 20.0, screen.h - 15.0, 15.0, 64)

  for i = 1, #balls do
      local ball = balls[i]
      love.graphics.setColor(255, 0, 0)
      love.graphics.circle("fill", ball.x, ball.y, ball.radius, 64)
  end
end
